@@ -28,7 +28,7 @@ from steam_nb_api.utils.SelfMutualInductanceCalculation import SelfMutualInducta
 from steam_nb_api.roxie_parser import CableDatabase
 from steam_nb_api.roxie_parser import ConductorPosition
 from steam_nb_api.roxie_parser.geometricFunctions import close_pairs_ckdtree, close_pairs_pdist
-from steam_nb_api.resources.ResourceReader import ResourceReader
+from steam_nb_api.resources.ResourceReader import *
 
 @dataclass
 class MagnetGeometry:
@@ -52,22 +52,22 @@ class MagnetField:
     Bx: np.ndarray = np.array([])
     By: np.ndarray = np.array([])
 
-def _read_yaml(type_str, elem_name):
-    """
-    Reads yaml file and returns it as dictionary
-    :param type_str: type of file, e.g.: quench, coil, wire
-    :param elem_name: file name, e.g. ColSol.1
-    :return: dictionary for file named: type.name.yam
-    """
-    file_path = os.path.join('ledet', 'Inputs', f"{type_str}.{elem_name}.yaml")
-    fullfileName = ResourceReader.getResourcePath(file_path)
-    with open(fullfileName, 'r') as stream:
-        data = yaml.safe_load(stream)
-    return data
+# def _read_yaml(type_str, elem_name):
+#     """
+#     Reads yaml file and returns it as dictionary
+#     :param type_str: type of file, e.g.: quench, coil, wire
+#     :param elem_name: file name, e.g. ColSol.1
+#     :return: dictionary for file named: type.name.yam
+#     """
+#     file_path = os.path.join('ledet', 'Inputs', f"{type_str}.{elem_name}.yaml")
+#     fullfileName = ResourceReader.getResourcePath(file_path)
+#     with open(fullfileName, 'r') as stream:
+#         data = yaml.safe_load(stream)
+#     return data
 
 class Notebook_LEDET:
     def __init__(self, nameMagnet, typeMagnet = 'Cos-Theta', verbose = False, recalc_field=True, headerLines =1):
-        self.AvailableConductor = ['Type2', 'W1']
+        self.AvailableConductor = ['Type2', 'W1', 'PSU1sn', 'PSU2ti']
         self.AvailableOptions = ['cos_theta', 'solenoid']
         self.AvailableTransients = ['FPA']
         self.AvailablePlotOptions = ['Default']
@@ -89,9 +89,14 @@ class Notebook_LEDET:
         self.selectedFont = {'fontname':'DejaVu Sans', 'size':14}
         self.text = {'x': [], 'y': [], 't': []}
         self.typeMagnet = typeMagnet
+        self.cart_labels = {'x': "x (m)", 'y': "y (m)"}
+        self.rad_labels = {'x': "r (m)", 'y': "z (m)"}
 
-        if typeMagnet == 'solenoid':
-            self.solenoid_data = _read_yaml('magnet', self.nameMagnet)
+        if self.typeMagnet == 'solenoid':
+            self.solenoid_data = read_yaml('magnet', self.nameMagnet)
+            self.out_dir = os.path.join(Path(os.getcwd()).parent, self.nameMagnet)
+            if not os.path.exists(self.out_dir):
+                os.makedirs(self.out_dir)
             self._setUp_Solenoid(recalc_field)
 
         # Start loading
@@ -101,7 +106,7 @@ class Notebook_LEDET:
         [self.MagnetField, self.MagnetGeometry] = self._load_ROXIE()
         self.elPairs_GroupTogether = np.array([])
 
-        if typeMagnet == 'solenoid':    # this can be only done after _load_ROXIE has been called.
+        if self.typeMagnet == 'solenoid':    # this can be only done after _load_ROXIE has been called.
             self.load_ConductorData(self.Solenoid.wires, self.Solenoid.wire_groups)
             max_distance = 0.5E-3
             self.set_ThermalConnections(max_distance)
@@ -120,8 +125,8 @@ class Notebook_LEDET:
         self.Solenoid = Solenoid_magnet(self.nameMagnet)
         self.Magnet.flagIron = self.solenoid_data['flagIron']
         self.Magnet.flagSelfField = self.solenoid_data['flagSelfField']
-        field_map_file = os.path.join(os.getcwd(), f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
-        Ind_matrix_file = os.path.join(os.getcwd(), f"{self.nameMagnet}_selfMutualInductanceMatrix.csv")
+        field_map_file = os.path.join(self.out_dir, f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
+        Ind_matrix_file = os.path.join(self.out_dir, f"{self.nameMagnet}_selfMutualInductanceMatrix.csv")
         if recalc_field:
             if self.Magnet.flagIron == 1 or self.Magnet.flagSelfField == 1:
                 raise Exception(
@@ -129,13 +134,24 @@ class Notebook_LEDET:
             else:
                 self.Solenoid.save_B_map(field_map_file)
                 self.Solenoid.save_L_M(Ind_matrix_file)
-        self.Magnet.Inputs.I00 = self.solenoid_data['I']
         self.Magnet.Inputs.T00 = self.solenoid_data['T']  # [K]
-
-        self.Magnet.Inputs.M_m = np.sum(np.loadtxt(Ind_matrix_file, delimiter=',', skiprows=1))
-        if not isinstance(self.Magnet.Inputs.M_m, np.ndarray):
-            self.Magnet.Inputs.M_m = np.array([self.Magnet.Inputs.M_m])
-        self.Magnet.Inputs.M_InductanceBlock_m = np.array([0])
+        if np.all(np.isclose(self.Solenoid.current, self.Solenoid.current[0])):
+            self.Magnet.Inputs.I00 = self.Solenoid.current[0]
+        else:
+            raise Exception("Issue with different current in each coil block, this might be okay, but never checked")
+        self.Magnet.Inputs.M_m = np.loadtxt(Ind_matrix_file, delimiter=',', skiprows=1)
+        seq = np.arange(self.Solenoid.section.size)
+        sec_uni = list(np.int_(np.unique(self.Solenoid.section)))
+        M_M = np.zeros_like(np.arange(len(sec_uni) ** 2).reshape(len(sec_uni), len(sec_uni)), dtype=np.float64)
+        for sec_n in sec_uni:
+            for sec_m in sec_uni:
+                idx_n = seq[np.isin(self.Solenoid.section, sec_n)]
+                idx_m = seq[np.isin(self.Solenoid.section, sec_m)]
+                chunk = self.Magnet.Inputs.M_m[np.ix_(idx_n, idx_m)]
+                res = np.sum(chunk)
+                M_M[sec_n - 1, sec_m - 1] = res
+        self.Magnet.Inputs.M_InductanceBlock_m = self.Magnet.Inputs.M_m
+        self.Magnet.Inputs.M_m = M_M
 
     def _load_ROXIE(self):
         # Select ROXIE .cadata file with conductor data
@@ -144,7 +160,7 @@ class Notebook_LEDET:
         fileName = self.nameMagnet + '_All_WithIron_WithSelfField.map2d'
         fileNameData = self.nameMagnet + '_All_WithIron_WithSelfField.data'
         if self.typeMagnet == 'solenoid' and self.solenoid_data['flagIron']==0 and self.solenoid_data['flagSelfField']==0:
-            fileName = self.nameMagnet + '_All_NoIron_NoSelfField.map2d'
+            fileName = os.path.join(self.out_dir, f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
 
         print('O - Loading field maps from: ', fileName)
         strandToGroup = np.array([])
@@ -274,7 +290,7 @@ class Notebook_LEDET:
             if not Conductor in self.AvailableConductor:
                 print('Conductor type: ',Conductor, ' not understood. Available conductor: ', self.AvailableConductor)
                 return
-            conductor_data = _read_yaml('conductor', Conductor)
+            conductor_data = read_yaml('conductor', Conductor)
             cable_data = conductor_data['cable']
             strand_data = conductor_data['strand']
             type = conductor_data['type']
@@ -350,7 +366,7 @@ class Notebook_LEDET:
             self.Magnet.Inputs.nT = []
             self.Magnet.Inputs.l_mag_inGroup = []
             for block_name, block_dict in self.solenoid_data['blocks'].items():
-                conductor_data = _read_yaml('conductor', block_dict['wire'])
+                conductor_data = read_yaml('conductor', block_dict['wire'])
                 block_dict['wire'] = conductor_data
                 sol = Solenoid(block_name, block_dict)
                 n_groups_in_block = np.shape(sol.Rin)[0]
@@ -374,8 +390,9 @@ class Notebook_LEDET:
             print('X - Calculation was successful.')
         elif self.typeMagnet == 'solenoid':
             self.nGroups = len(self.Magnet.Inputs.ds_inGroup)
-            self.Magnet.Inputs.GroupToCoilSection = self.nGroups * [1]
-            self.nCoilSections = np.max(self.Magnet.Inputs.GroupToCoilSection)
+            self.Magnet.Inputs.GroupToCoilSection = self.Solenoid.section
+            #self.Magnet.Inputs.GroupToCoilSection = self.nGroups * [1]
+            self.nCoilSections = np.int_(np.max(self.Magnet.Inputs.GroupToCoilSection))
             # Define to which inductive block each half-turn belongs
             self.Magnet.Inputs.HalfTurnToInductanceBlock = []
             for g in range(1, self.nGroups + 1):
@@ -394,7 +411,7 @@ class Notebook_LEDET:
             print('Option type ', Type, ' currently not supported or not understood. '
                                         'Available options: ', self.AvailableOptions)
             return
-        option_data = _read_yaml('options', Type)
+        option_data = read_yaml('options', Type)
         for key in option_data.keys():
             if key in self.Magnet.Options.__annotations__:
                 Values = option_data[key]
@@ -410,7 +427,7 @@ class Notebook_LEDET:
             print('Storage type ', Type, ' currently not supported or not understood '
                                         'Available options: ', self.AvailableVariables)
             return
-        option_data = _read_yaml('store', Type)
+        option_data = read_yaml('store', Type)
         for key in option_data.keys():
             if key in self.Magnet.Variables.__annotations__:
                 Values = option_data[key]
@@ -427,7 +444,7 @@ class Notebook_LEDET:
             print('Plot-options type ', Type, ' currently not supported or not understood.'
                                          'Available plot-options: ', self.AvailablePlotOptions)
             return
-        option_data = _read_yaml('plots', Type)
+        option_data = read_yaml('plots', Type)
         for key in option_data.keys():
             if key in self.Magnet.Variables.__annotations__:
                 Values = option_data[key]
@@ -736,6 +753,7 @@ class Notebook_LEDET:
                 for i in range(self.Magnet.Inputs.M_InductanceBlock_m.shape[0]):
                     reader.writerow(self.Magnet.Inputs.M_InductanceBlock_m[i])
             self.Magnet.Inputs.M_InductanceBlock_m = np.array([0])
+        print('a')
 
     def set_QuenchHeater(self, nHalfQuadrants, iQH_toHalfTurn_From_oneHalfQuadrant, iQH_toHalfTurn_To_oneHalfQuadrant):
         idxHQ = int(self.nGroups/nHalfQuadrants)
@@ -795,33 +813,36 @@ class Notebook_LEDET:
         print('X - Variables successfully loaded.')
 
     def _solenoid_locals(self):
-        scenario_data = _read_yaml('scenario', self.nameMagnet)
-        circuit_data = _read_yaml('circuit', scenario_data['circuit'])
-        protection_data = _read_yaml('protection', scenario_data['protection'])
+        magnet_data = read_yaml('magnet', self.nameMagnet)
+        circuit_data = read_yaml('circuit', magnet_data['circuit'])
+        protection_data = read_yaml('protection', magnet_data['protection'])
 
-        R_EE_triggered = (protection_data['U_EE'] - self.solenoid_data['I'] * (
-                circuit_data['R_circuit'] + circuit_data['R_crowbar']) - circuit_data['Ud_crowbar']) / self.solenoid_data['I']  # Resistance of the energy-extraction system [Ohm]
+        R_EE_triggered = (protection_data['U_EE'] - self.Magnet.Inputs.I00 * (
+                circuit_data['R_circuit'] + circuit_data['R_crowbar']) - circuit_data['Ud_crowbar']) / self.Magnet.Inputs.I00  # Resistance of the energy-extraction system [Ohm]
         if R_EE_triggered < 0:
             R_EE_triggered = 0
         time_v_array = [item for sublist in [[rng['start'], rng['step'], rng['stop']] for rng in
-                        scenario_data['simul_time']] for item in sublist]
+                        magnet_data['simul_time']] for item in sublist]
         dict_block = {
         'R_EE_triggered': R_EE_triggered,
         'alphasDEG': self.nHalfTurns * [0],
         'rotation_block': self.nHalfTurns * [0],
         'mirror_block': self.nHalfTurns * [0],
         'mirrorY_block': self.nHalfTurns * [0],
-        'initialQuenchTemp': 10,
+        #'initialQuenchTemp': 10,
+        'initialQuenchTemp': [1.],
         'tCLIQ': 99999,                  # Time when the CLIQ system is triggered [s]
-        'directionCurrentCLIQ': [1],    # Direction of the introduced current change for the chosen CLIQ configuration
+        'directionCurrentCLIQ': [1]* self.nCoilSections,    # Direction of the introduced current change for the chosen CLIQ configuration
         'nCLIQ': 0,                     # Number of CLIQ units
         'U0': 0,                        # CLIQ charging voltage [V]
         'C': 0,                         # Capacitance of the CLIQ capacitor bank [F]
         'Rcapa': 0,                      # Resistance of the CLIQ leads [Ohm]
         'fL_I': [0, 1000],
         'fL_L': [1, 1],
-        'tQuench': scenario_data['tStartQuench'],
-        'tStart': time_v_array[0],
+        'tStartQuench': magnet_data['tStartQuench'],
+        'tQuench': magnet_data['tStartQuench'],
+        'iStartQuench': magnet_data['iStartQuench'],
+        'tStart': time_v_array[0],      # this is needed for power supply LUT
         'time_vector_params': time_v_array
         }
         return {**circuit_data, **protection_data, **dict_block}
@@ -829,6 +850,7 @@ class Notebook_LEDET:
     def writeLEDETFile(self, nameFileLEDET, locals):
         if self.typeMagnet == 'solenoid':
             locals = {**locals, **self._solenoid_locals()}
+            nameFileLEDET = os.path.join(self.out_dir, nameFileLEDET)
         self.storeVariables(locals)
         # Set flags depending on content:
         if len(self.ConductorTypes)>1 and self.Magnet.Options.flag_hotSpotTemperatureInEachGroup != 1:
@@ -843,6 +865,34 @@ class Notebook_LEDET:
 
         self.Magnet.writeFileLEDET(nameFileLEDET)
 
+    def write_start_bat(self, model_no):
+        script_lines = ['for %%a in ("%~p0../../..") do set version=%%~nxa',
+                        'pushd ..',
+                        'pushd ..',
+                        'for %%x in ("%CD%") do set folder=%%~dpxLEDET',
+                        'pushd ..',
+                        'for %%x in ("%CD%") do set ex=%%~dpx%version%\%version%.exe',
+                        'cd %~dp0',
+                        f'start %ex% %folder% {self.nameMagnet} {int(model_no)}']
+        batch_file_path = os.path.join(self.out_dir, f"{self.nameMagnet}_{int(model_no)}.bat")
+        with open(batch_file_path, "w") as outfile:
+            outfile.write("\n".join(str(line) for line in script_lines))
+
+    def run_LEDET_bat_parallel(self, model_no):
+        os.chdir(self.magnet_folder)
+        call_string = f"{self.magnet_name}_{int(model_no)}.bat"
+        subprocess.call([call_string])
+
+    def run_LEDET_bat_sequentially(self, model_no):
+        os.chdir(self.magnet_folder)
+        # LEDET_exe_path = os.path.join(LEDET_dir, LEDET_exe)
+        batch_file_path = os.path.join(os.getcwd(), f"{self.nameMagnet}_{model_no}.bat")
+        p = subprocess.Popen(batch_file_path, shell=False,
+                             stdout=subprocess.PIPE)
+        stdout, stderr = p.communicate()
+        print(p.returncode)
+
+
     ## Plotting part
     def plotter(self, data, titles, labels, types, texts, size=()):
         """
@@ -884,7 +934,12 @@ class Notebook_LEDET:
         ConductorGroups = np.repeat(ConductorGroups, nStrands.astype(int))
         data = [{'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': ConductorGroups}]
         titles = ['Conductor Type']
-        labels = [{'x': "x (m)", 'y': "y (m)", 'z': ""}] * len(data)
+        label = {'z': ""}
+        if self.typeMagnet == 'solenoid':
+            label.update(self.rad_labels)
+        else:
+            label.update(self.cart_labels)
+        labels = [label] * len(data)
         types = ['scatter'] * len(data)
         texts = [self.text] * len(data)
         self.plotter(data, titles, labels, types, texts)
@@ -902,7 +957,12 @@ class Notebook_LEDET:
                 {'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': self.MagnetField.By},
                 {'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': self.MagnetField.B}]
         titles = ['Current [A]', 'By [T]', 'Bz [T]', 'Bmod [T]']
-        labels = [{'x': "x (m)", 'y': "y (m)", 'z': ""}] * len(data)
+        label = {'z': ""}
+        if self.typeMagnet == 'solenoid':
+            label.update(self.rad_labels)
+        else:
+            label.update(self.cart_labels)
+        labels = [label] * len(data)
         types = ['scatter'] * len(data)
         texts = [self.text] * len(data)
         self.plotter(data, titles, labels, types, texts)
@@ -914,10 +974,15 @@ class Notebook_LEDET:
                 {'x': self.MagnetGeometry.x_ave, 'y': self.MagnetGeometry.y_ave, 'z': self.halfTurnToTurn},
                 {'x': self.MagnetGeometry.x_ave, 'y': self.MagnetGeometry.y_ave, 'z': np.repeat(self.Magnet.Inputs.nStrands_inGroup, self.Magnet.Inputs.nT)}]
         titles = ['strandToHalfTurn', 'strandToGroup', 'halfTurnToTurn', 'Number of strands per half-turn']
-        labels = [{'x': "r (m)", 'y': "z (m)", 'z': "Half-turn [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Group [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Turn [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Number of  strands per cable [-]"}]
+        labels = [{'z': "Half-turn [-]"},
+                  {'z': "Group [-]"},
+                  {'z': "Turn [-]"},
+                  {'z': "Number of  strands per cable [-]"}]
+        for label in labels:
+            if self.typeMagnet == 'solenoid':
+                label.update(self.rad_labels)
+            else:
+                label.update(self.cart_labels)
         t_ht = copy.deepcopy(self.text)
         for ht in range(self.nHalfTurns):
             t_ht['x'].append(self.MagnetGeometry.x_ave[ht])
@@ -938,7 +1003,12 @@ class Notebook_LEDET:
             polarities_inStrand[np.where(self.strandToGroup == g)] = self.Magnet.Inputs.polarities_inGroup[g - 1]
         data = [{'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': polarities_inStrand}]
         titles = ['Current polarities']
-        labels = [{'x': "r (m)", 'y': "z (m)", 'z': "Polarity [-]"}]
+        labels = [{'z': "Polarity [-]"}]
+        for label in labels:
+            if self.typeMagnet == 'solenoid':
+                label.update(self.rad_labels)
+            else:
+                label.update(self.cart_labels)
         types = ['scatter'] * len(data)
         texts = [self.text] * len(data)
         self.plotter(data, titles, labels, types, texts)
@@ -949,10 +1019,15 @@ class Notebook_LEDET:
                 {'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': self.strandToGroup},
                 {'x': self.MagnetGeometry.x, 'y': self.MagnetGeometry.y, 'z': self.strandToCoilSection}]
         titles = ['HalfTurnToGroup', 'HalfTurnToCoilSection', 'StrandToGroup', 'StrandToCoilSection']
-        labels = [{'x': "r (m)", 'y': "z (m)", 'z': "Group [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Coil section [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Group [-]"},
-                  {'x': "r (m)", 'y': "z (m)", 'z': "Coil Section [-]"}]
+        labels = [{'z': "Group [-]"},
+                  {'z': "Coil section [-]"},
+                  {'z': "Group [-]"},
+                  {'z': "Coil Section [-]"}]
+        for label in labels:
+            if self.typeMagnet == 'solenoid':
+                label.update(self.rad_labels)
+            else:
+                label.update(self.cart_labels)
         types = ['scatter'] * len(data)
         texts = [self.text] * len(data)
         self.plotter(data, titles, labels, types, texts)
@@ -1204,13 +1279,15 @@ class Notebook_LEDET:
 
     def plot_all(self):
         self.plot_field()
-        self.plot_Conductors()
-        #self.plot_polarities()
+        #self.plot_Conductors()
+        self.plot_polarities()
         self.plot_strands_groups_layers()
         self.plot_quench_prop_and_resist()
         self.plot_half_turns()
         #self.plot_electrical_order()
         self.plot_heat_exchange_order()
         self.plot_nonlin_induct()
-        self.plot_psu_and_trig()
-        self.plot_power_supl_contr()
+        #self.plot_psu_and_trig()
+        #self.plot_power_supl_contr()
+
+
