@@ -137,7 +137,7 @@ class ParameterSweep(object):
         LUT[LUT != 0] = currentLvl
         self.ParametersLEDET.setAttribute(getattr(self.ParametersLEDET, 'Inputs'), 'I_PC_LUT', LUT)
 
-    def prepareSimulation(self, MagnetName, folder, cleanFolder = False,  OffsetNumber = 0, ROXIE_File = ''):
+    def prepareSimulation(self, MagnetName, folder, cleanFolder = False,  OffsetNumber = 0, ROXIE_File = '', UseMat = 0):
         Adjust_vQ = 0
         if ROXIE_File:
             Adjust_vQ =1
@@ -179,7 +179,7 @@ class ParameterSweep(object):
             if Adjust_vQ:
                 self.ParametersLEDET.adjust_vQ(ROXIE_File)
 
-            self.ParametersLEDET.Options.flag_saveMatFile = 0
+            self.ParametersLEDET.Options.flag_saveMatFile = UseMat
             self.ParametersLEDET.Options.flag_generateReport = 0
             self.ParametersLEDET.writeFileLEDET(file_name)
         if 'I_CoilSections' not in self.ParametersLEDET.Variables.variableToSaveTxt:
