@@ -1,4 +1,5 @@
 import os
+from pathlib import Path
 import numpy as np
 import datetime
 import csv
@@ -69,7 +70,7 @@ class LEDETInputs:
     Ud_crowbar: float = 0.0
     tEE: float = 9999
     R_EE_triggered: float = 0.0
-    tCLIQ: np.ndarray = np.array([9999])
+    tCLIQ: np.ndarray = 9999
     directionCurrentCLIQ: np.ndarray = np.array([0])
     nCLIQ: np.ndarray = np.array([0])
     U0: np.ndarray = np.array([0])
@@ -600,6 +601,22 @@ class ParametersLEDET:
         rhoCu[idxHighB] = (1. + corrs * f_MR) * rhcu[idxHighB]
         return rhoCu
 
+    def _rhoSS(self, T):
+        LimitValidityLow = 0
+        LimitValidityHigh = 300
+
+        fit_rho_SS_CERN = np.array([-6.16E-15, 3.52E-12, 1.72E-10, 5.43E-07]) / 1.0867
+        fit_rho_SS_CERN_linearExtrapolation = np.array([7.24E-10, 5.2887E-7]) / 1.0867
+
+        rhoSS = 0
+        if T < LimitValidityLow:
+            rhoSS = np.polyval(fit_rho_SS_CERN, LimitValidityLow)
+        elif T >= LimitValidityLow and T <= LimitValidityHigh:
+            rhoSS = np.polyval(fit_rho_SS_CERN, T)
+        elif T > LimitValidityHigh:
+            rhoSS = np.polyval(fit_rho_SS_CERN_linearExtrapolation, T)
+        return rhoSS
+
     def __kCu_WiedemannFranz(self, rhoCu, T):
         L0 = 2.45E-8
         kCu = L0 * T / rhoCu
@@ -612,9 +629,9 @@ class ParametersLEDET:
         idxT2 = np.where(T > LimitValidity)
 
         a, b, c, d, e, f, g, h = -4.1236, 13.788, -26.068, 26.272, -14.663, 4.4954, -0.6905, 0.0397
-        logT = np.log10(T[idxT1]);
+        logT = np.log10(T[idxT1])
         logk = a + b * logT + c * logT**2 + d * logT**3 + e * logT**4 + f * logT**5 + g * logT**6 + h * logT**7
-        kG10[idxT1] = 10**logk;
+        kG10[idxT1] = 10**logk
 
         logLimitValidity = np.log10(LimitValidity)
         logkLimitValidity = a + b * logLimitValidity + c * logLimitValidity**2 + d * logLimitValidity**3 + e * logLimitValidity**4 + \
@@ -814,7 +831,7 @@ class ParametersLEDET:
                 delta_t_w_temp.append(delta_t)
             delta_t_w[str(i)] = delta_t_w_temp
 
-        return [th_con_h, delta_t_h, th_con_w, delta_t_w]
+        return [HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w]
 
     def _quenchPropagationVelocity(self, I, B, T_bath, cable):
         # Calculate Quench propagation velocity
@@ -871,8 +888,8 @@ class ParametersLEDET:
         cp_full = (cp* (A_CableBare/cable.A_CableInsulated) + cpIns*(1-A_CableBare/cable.A_CableInsulated))/2
 
         ### Calculate delta T transversal
-        [th_con_h, delta_t_h, th_con_w, delta_t_w] = self.__calculateTransversalDelay(cp_full, kIns, Tc, Tcs, T_bath)
-        return [vQ, l, th_con_h, delta_t_h, th_con_w, delta_t_w]
+        [HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w] = self.__calculateTransversalDelay(cp_full, kIns, Tc, Tcs, T_bath)
+        return [vQ, l, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w]
 
     def __reorderROXIEFiles(self, ROXIE_File):
         orderedROXIE = []
@@ -898,6 +915,7 @@ class ParametersLEDET:
                 ROXIE_File1[i] = os.path.join(ROXIE_File, ROXIE_File1[i])
             ROXIE_File = ROXIE_File1
             ROXIE_File = self.__reorderROXIEFiles(ROXIE_File)
+
             N = len(ROXIE_File)
             if N>1:
                 print('Reading ', N, ' Field maps. This may take a while.')
@@ -952,7 +970,73 @@ class ParametersLEDET:
             setattr(newCable, attribute, x)
         return newCable
 
-    def adjust_vQ(self, ROXIE_File, Return  = False):
+    def calculateQuenchDetectionTime(self, Type, B, vQ_iStartQuench, lengthHotSpot_iStartQuench, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w, uQuenchDetectionThreshold = 0.1):
+        if not (Type=='Short' or Type=='Long'):
+            print("Don't understand type of quench detection time calculation. Please choose either 'Short' or 'Long'[incl. transversal propagation]")
+            print("Let's continue with 'Short'.")
+            Type = 'Short'
+        # Calculate resistance of each turn at T=10 K
+        rho_Cu_10K = 1.7E-10  # [Ohm*m] Approximate Cu resistivity at T=10 K, B=0, for RRR=100
+        rho_Cu_10K_B = 4E-11  # [Ohm*m/T] Approximate Cu magneto-resistivity factor
+        Iref = self.Options.Iref
+        nStrands_inGroup = self.Inputs.nStrands_inGroup
+        ds_inGroup = self.Inputs.ds_inGroup
+        f_SC_strand_inGroup = self.Inputs.f_SC_strand_inGroup
+        nHalfTurns = len(vQ_iStartQuench)
+
+        tQuenchDetection = []
+        r_el_m = np.zeros((nHalfTurns,))
+        for ht in range(1, nHalfTurns + 1):
+            current_group = HalfTurnToGroup[ht - 1]
+            mean_B = B / Iref * self.Inputs.I00  # average magnetic field in the current half-turn
+            rho_mean = rho_Cu_10K + rho_Cu_10K_B * mean_B[ht-1]  # average resistivity in the current half-turn
+            cross_section = nStrands_inGroup[current_group - 1] * np.pi / 4 * ds_inGroup[current_group - 1] ** 2 * (1 - f_SC_strand_inGroup[current_group - 1])
+            r_el_m[ht - 1] = rho_mean / cross_section # Electrical resistance per unit length
+            if Type == 'Short':
+                UQD_i = (self.Inputs.I00 * r_el_m[ht - 1] * lengthHotSpot_iStartQuench[ht - 1])
+                tQD = (uQuenchDetectionThreshold - UQD_i) / (vQ_iStartQuench[ht - 1] * r_el_m[ht - 1] * self.Inputs.I00)
+                tQuenchDetection = np.hstack([tQuenchDetection, np.array(tQD)])
+
+        r_el_m = r_el_m.transpose()
+        if Type == 'Long':
+            for ht in range(1, nHalfTurns + 1):
+                for ht in range(1, nHalfTurns + 1):
+                    # Approximate time to reach the quench detection threshold
+                    UQD_i = (self.Inputs.I00 * r_el_m[ht - 1] * lengthHotSpot_iStartQuench[ht - 1])
+                    tQD = (uQuenchDetectionThreshold - UQD_i) / (vQ_iStartQuench[ht - 1] * r_el_m[ht - 1] * self.Inputs.I00)
+                    delay = np.concatenate((np.array(delta_t_w[str(ht)]), np.array(delta_t_h[str(ht)])), axis=None)
+                    th_con = np.concatenate((np.array(th_con_w[str(ht)]), np.array(th_con_h[str(ht)])), axis=None).astype(int)
+                    tQD_i = tQD
+                    t_i0 = 0
+                    t_i1 = 0
+                    idx_turns = np.array([ht - 1])
+                    quenched_turns = [ht]
+                    delay[delay > tQD_i] = 9999
+
+                    while np.any(delay < 999):
+                        idx = np.argmin(delay)
+                        if th_con[idx] in quenched_turns:
+                            delay[idx] = 9999
+                            continue
+                        else:
+                            quenched_turns.append(int(th_con[idx]))
+                        UQD_i = UQD_i + np.sum(self.Inputs.I00 * r_el_m[idx_turns] * (t_i1 - t_i0) * vQ_iStartQuench[idx_turns])
+                        idx_turns = np.append(idx_turns, int(th_con[idx] - 1))
+                        t_i1 = delay[idx]
+                        tQD_i = (uQuenchDetectionThreshold - UQD_i) / (
+                                np.sum(vQ_iStartQuench[idx_turns] * r_el_m[idx_turns] * self.Inputs.I00))
+                        t_i0 = t_i1
+                        delay = np.concatenate((delay, np.array(delta_t_w[str(int(th_con[idx]))] + t_i1),
+                                                    np.array(delta_t_h[str(int(th_con[idx]))] + t_i1)), axis=None)
+                        th_con = np.concatenate((th_con, np.array(th_con_w[str(int(th_con[idx]))]),
+                                                 np.array(th_con_h[str(int(th_con[idx]))])),axis=None)
+                        delay[delay > tQD_i] = 9999
+                        delay[idx] = 9999
+                tQuenchDetection = np.hstack([tQuenchDetection, np.array(tQD_i)])
+        print('Minimum quench detection time would be {} ms [{} calculation]'.format(round(min(tQuenchDetection),3)*1000, Type))
+        return min(tQuenchDetection)
+
+    def adjust_vQ(self, ROXIE_File, Transversaldelay  = False):
         cable = Cable()
         cable.A_CableInsulated = (self.Inputs.wBare_inGroup+2*self.Inputs.wIns_inGroup) \
                                * (self.Inputs.hBare_inGroup+2*self.Inputs.hIns_inGroup)
@@ -1002,11 +1086,13 @@ class ParametersLEDET:
         cable.Bc20_Nb3Sn = self.Inputs.Bc2_Nb3Sn_inGroup
         cable = self.__repeatCable(cable)
 
-        [vQ, l, th_con_h, delta_t_h, th_con_w, delta_t_w] = self._quenchPropagationVelocity(I, B, T_bath, cable)
+        [vQ, l, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w] = self._quenchPropagationVelocity(I, B, T_bath, cable)
         self.setAttribute(getattr(self, "Inputs"), "vQ_iStartQuench", vQ)
         #self.setAttribute(getattr(self, "Inputs"), "lengthHotSpot_iStartQuench", l)
-        if Return:
-            return [vQ, l, th_con_h, delta_t_h, th_con_w, delta_t_w]
+        if Transversaldelay:
+            tQD = self.calculateQuenchDetectionTime(Transversaldelay, B, vQ, l, HalfTurnToGroup, th_con_h, delta_t_h,
+                                              th_con_w, delta_t_w, uQuenchDetectionThreshold = 0.1)
+            return [l, tQD]
         else:
             return
 
