@@ -3,14 +3,12 @@ import matplotlib.pyplot as plt
 import matplotlib.cm as cm
 from matplotlib import colors as mcolors
 import h5py
-import copy
 from steam_nb_api.ledet.ParameterSweep import *
 from nptdms import TdmsFile
 import nptdms
 from scipy import signal
 import csv
 from scipy.integrate import simps
-import time
 
 
 @dataclass
@@ -20,11 +18,17 @@ class TDMSdata:
     groupHF:  nptdms.GroupObject = nptdms.GroupObject('HF')
     groupMF: nptdms.GroupObject = nptdms.GroupObject('MF')
     VoltageVertices: np.ndarray = np.array([])
+    # Times
     TimeFrame_MF: np.ndarray = np.array([])
     t_steps_MF:  float = 0.0
     t_d_MF: float = 0.0
     trigger_PC: int = 0
     trigger_shoot: int = 0
+    t_CLIQ: np.ndarray = np.array([])
+    t_QH: np.ndarray = np.array([])
+    t_PC: np.ndarray = np.array([])
+    t_EE: np.ndarray = np.array([])
+    #Electric
     I_DCCT_MF: np.ndarray = np.array([])
     I_DCCT_HF: np.ndarray = np.array([])
     U_PC: np.ndarray = np.array([])
@@ -34,12 +38,18 @@ class TDMSdata:
     f_CLIQ: np.ndarray = np.array([])
     I_QH: np.ndarray = np.array([])
     U_QH: np.ndarray = np.array([])
+    R_QH: np.ndarray = np.array([])
+    C_QH: np.ndarray = np.array([])
+    tau_QH: np.ndarray = np.array([])
+    U_EE: np.ndarray = np.array([])
+    R_EE: float = 0.0
+    R_crow: float = 0.0
+    R_circuit: float = 0.0
+    Ud_crow: float = 0.0
+    #Postprocess
     QL1: float = 0.0
     QL2: float = 0.0
     Quenches: np.ndarray = np.array([])
-    R_crow: float = 0.0
-    R_circuit: float = 0.0
-    U_TH: float = 0.0
 
 @dataclass
 class SIMdata:
@@ -48,6 +58,7 @@ class SIMdata:
     trigger_PC: int = 0
     XY_mag_ave: np.ndarray = np.array([])
     I_CoilSections: np.ndarray = np.array([])
+    Uturn_half_turns: np.ndarray = np.array([])
     I_CLIQ: np.ndarray = np.array([])
     directionsCLIQ: np.ndarray = np.array([])
     U_CLIQ: np.ndarray = np.array([])
@@ -70,11 +81,10 @@ class WrappedVoltages:
 
 class QuenchPlanAnalysis():
     ## Domain: Helper and init function
-    def __init__(self, FileNameTDMS, FileNameSIMData, beforePC = 0.4, verbose =  False):
+    def __init__(self, FileNameTDMS, FileNameSIMData, beforePC = 0.4, verbose: bool = False):
         self.beforePC = beforePC
         self.Colors = self.__generateColors()
         self.verbose = verbose
-
         self.MergedVoltages = {}
         self.MergedVoltagesPolarities = {}
         self.MergedVoltagesValues = {}
@@ -148,7 +158,7 @@ class QuenchPlanAnalysis():
             idxQH = np.append(idxQH, [int(x)-1 for x in QHpairs[int(numbers[i])-1]])
         return idxQH.astype(int)
 
-    def PrepareSimulations(self, PlanFile, RefFile, OutputDirectory, MagnetName = '', QHpairs = [[]], nQHseries = 2,
+    def PrepareSimulations(self, PlanFile, RefFile, OutputDirectory, MagnetName = '', QHpairs = [[]],
                            t_QH_Fire = 0.002, t_CLIQ_Fire = 0.0005, t_EE_Fire = 0.008):
         ## TODO Detection and Validation times!
 
@@ -229,11 +239,17 @@ class QuenchPlanAnalysis():
                     a.setAttribute("Inputs", "tEE", 9999)
 
                 ### 4. Adjust Quench Heater
+                if QHpairs:
+                    nQHseries = len(QHpairs[0])
+                    for i in range(len(QHpairs)):
+                        if len(QHpairs[i]) != nQHseries:
+                            print("Provided Quench Heater pairs are not symmetric. I'm not sure how to handle this.")
+                else: nQHseries = 2
                 if df.iloc[index]['QH'] == df.iloc[index]['QH']:
                     nQH = len(a.getAttribute("Inputs", "tQH"))
                     if len(QHpairs) == 1:
                         if self.verbose: print('No Quench Heater pairs provided. Set all QH to be handled together.')
-                        QHpairs[0] = np.linspace(0,nQH-1, nQH).astype(int).tolist()
+                        QHpairs[0] = np.linspace(0, nQH-1, nQH).astype(int).tolist()
 
                     V_QH = [df.iloc[index]['V_QH'] / nQHseries] * nQH
                     a.setAttribute("Inputs", "U0_QH", V_QH)
@@ -261,6 +277,89 @@ class QuenchPlanAnalysis():
                 SimNumbers.append(index)
         return
 
+    def PrepareSimulation_FromMeasurements(self, RefFile, OutputDirectory, MagnetName = '', QHpairs = [[]], nQHseries = 0):
+        if not self.TDMSdata.FileName:
+            print('Please first provide measured data. Abort.')
+            return
+        # Find MagnetName in RefFile Name
+        if not MagnetName:
+            RFile = RefFile.replace('\\', '//')
+            idxRef = self.__findOccurrences(RFile, '/')
+            idxRef = idxRef[-1] + 1
+            idxScore = RFile.find('_')
+            MagnetName = RFile[idxRef:idxScore]
+        if self.verbose: print("MagnetName = ",MagnetName)
+
+        a = ParametersLEDET()
+        a.readLEDETExcel(RefFile)
+
+        # 1. Adjust current in file
+        I_init = self.ExtractFeatures(TDMS = 1, SIM = 0)
+        a.Inputs.I00 = I_init
+        I_LUT = np.array([I_init, I_init, 0])
+        t_LUT = np.array([-0.02, self.TDMSdata.t_PC, self.TDMSdata.t_PC+0.01])
+        a.Inputs.t_PC_LUT = t_LUT
+        a.Inputs.I_PC_LUT = I_LUT
+        a.Inputs.t_PC = self.TDMSdata.t_PC
+
+        # 2. Adjust Electrical parameters
+        a.Inputs.R_circuit = self.TDMSdata.R_circuit
+        a.Inputs.Ud_crowbar = self.TDMSdata.Ud_crow
+
+        # 2.1 Adjust Quench Heater
+        if np.any(self.TDMSdata.t_QH):
+            if not QHpairs[0] and nQHseries == 0:
+                print('Quench Heater shall be active but I got no information about them.')
+                print('Please either provide the arrangement of the heaters as QHpairs = [List of lists] or nQHseries = Number of heater strips in series.')
+                print('Abort.')
+                return
+            elif not QHpairs[0]:
+                for i in range(len(self.TDMSdata.t_QH)):
+                    QHapp = []
+                    for j in range(nQHseries):
+                        QHapp = QHapp+[i*nQHseries+1+j]
+                    if i==0: QHpairs[0] = QHapp
+                    else: QHpairs.append(QHapp)
+            elif not nQHseries:
+                nQHseries = len(QHpairs[0])
+                for i in range(len(QHpairs)):
+                    if len(QHpairs[i]) != nQHseries:
+                        print("Provided Quench Heater pairs are not symmetric. I'm not sure how to handle this.")
+            U0_QH = np.max(self.TDMSdata.U_QH,axis=1)
+            rhoSS = a._rhoSS(a.Inputs.T00)
+            R_cold_QH = (rhoSS /(a.Inputs.w_QH * a.Inputs.h_QH)) * a.Inputs.l_QH * a.Inputs.f_QH
+
+            for i in range(len(self.TDMSdata.t_QH)):
+                idx_QH = QHpairs[i]
+                idx_QH = [k-1 for k in idx_QH]
+                a.Inputs.tQH[idx_QH] = self.TDMSdata.t_QH[i]
+                a.Inputs.U0_QH[idx_QH] = U0_QH[i]/nQHseries
+                a.Inputs.C_QH[idx_QH] = self.TDMSdata.C_QH[i]*nQHseries
+                R_warm_QH = (self.TDMSdata.R_QH[i] / nQHseries) - R_cold_QH[idx_QH[0]]
+                a.Inputs.R_warm_QH[idx_QH] = R_warm_QH
+        else:
+            a.Inputs.tQH = [9999]*len(a.Inputs.tQH)
+
+        # 2.2. Adjust CLIQ
+        if not np.isnan(self.TDMSdata.t_CLIQ):
+            a.Inputs.tCLIQ = self.TDMSdata.t_CLIQ[0]
+            a.Inputs.C = self.TDMSdata.C_CLIQ
+            a.Inputs.U0 = np.amax(self.TDMSdata.U_CLIQ)
+            for i in range(len(a.Inputs.directionCurrentCLIQ)):
+                idx_cs = np.where(a.Inputs.GroupToCoilSection == i+1)[0]
+                ## TODO: That's not really right [-1*]
+                a.Inputs.directionCurrentCLIQ[i] = -1* a.Inputs.polarities_inGroup[idx_cs]
+            ## TODO: R_CLIQ: See Notebook for attenuation calculation
+        else:
+            a.Inputs.tCLIQ = np.array([9999])
+
+        # 2.3 Adjust EE
+        if self.TDMSdata.t_EE:
+            a.Inputs.tEE = self.TDMSdata.t_EE
+            a.Inputs.R_EE_triggered = self.TDMSdata.R_EE
+
+        a.writeFileLEDET(os.path.join(OutputDirectory, MagnetName + '_' + str(0) + '.xlsx'))
+
     #############################################################################################################
     ## Domain: Functions used by both SIMdata/TMDSdata
     def __zero_runs(self, I, Itol = 10):
@@ -306,7 +405,9 @@ class QuenchPlanAnalysis():
             ax.plot(TimeFrame, I_CLIQ)
             ax.grid(True)
         if len(f_CLIQ)<2: F_CYCLE = 0
-        return f_CLIQ[F_CYCLE]
+
+        if f_CLIQ.size==0: return 0
+        else: return f_CLIQ[F_CYCLE]
 
     def _CalculateCLIQCapacitance(self, TimeFrame, I_CLIQ, U_CLIQ, kernel_size, I_tol = 50, Plot = 0):
         filtered = signal.medfilt(U_CLIQ[:, 0], kernel_size=kernel_size)
@@ -393,7 +494,7 @@ class QuenchPlanAnalysis():
         return
 
     def GetTDMSCoilStructure(self):
-        print('*** Provided eletrical connections in TDMS data ***')
+        print('*** Provided electrical connections in TDMS data ***')
         CoilCount = 0
         for i in range(len(self.TDMSdata.VoltageVertices)):
             temp = self.TDMSdata.VoltageVertices[i][1]
@@ -534,6 +635,7 @@ class QuenchPlanAnalysis():
             if len(self.TDMSdata.U_PC.shape) < 2: self.TDMSdata.U_PC = np.array([self.TDMSdata.U_PC]).transpose()
             if channel.name.startswith("I_Heater") or channel.name.startswith("I(Alim__QH_"): I_Heaters.append(channel.name)
             if channel.name.startswith("U_Heater") or channel.name.startswith("U(Alim_QH_"): U_Heaters.append(channel.name)
+            if channel.name == 'Vdump': self.TDMSdata.U_EE = channel.data
         ## Set up Timeframe
         self.TDMSdata.TimeFrame_MF = np.array([np.linspace(0,self.TDMSdata.t_steps_MF*self.TDMSdata.t_d_MF, int(self.TDMSdata.t_steps_MF))]).transpose()
         #self.TDMSdata.goupHF['IDCCT_HF'].time_track()
@@ -551,6 +653,7 @@ class QuenchPlanAnalysis():
         if self.TDMSdata.f_CLIQ > 0:
             self.TDMSdata.C_CLIQ = self._CalculateCLIQCapacitance(self.TDMSdata.TimeFrame_MF, self.TDMSdata.I_CLIQ,
                                                                   self.TDMSdata.U_CLIQ, 17)
+        self.TDMSdata.U_EE = self.TDMSdata.U_EE[trigger_shoot-1:]
         self.__FillHeaterArrays(I_Heaters, trigger_shoot)
         self.__FillHeaterArrays(U_Heaters, trigger_shoot)
         self.__CreateVoltageVertices()
@@ -780,58 +883,103 @@ class QuenchPlanAnalysis():
         # Extract R_Circuit
         U_init = sum(self.TDMSdata.U_PC[:self.TDMSdata.trigger_PC, 0])/self.TDMSdata.trigger_PC
         self.TDMSdata.R_circuit = U_init/I_init
-        if self.verbose: print('Measured lead resistance: ',self.TDMSdata.R_circuit)
+        if self.verbose: print('Measured lead resistance: ', self.TDMSdata.R_circuit)
 
-        # Find Ud_crowbar
-        I_th = signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 101)
+        # Find Ud_crowbar & R_crowbar
+        I_th = signal.medfilt(self.TDMSdata.I_DCCT_MF[:, 0], kernel_size = 101)
         I_th[abs(I_th) < I_th_tol] = 0
         ranges = self.__zero_runs(I_th)[-1][0]
-
         idx_add = int(0.05/self.TDMSdata.t_d_MF)
         self.TDMSdata.Ud_crow = abs(np.sum(self.TDMSdata.U_PC[ranges:ranges+idx_add,0])/len(self.TDMSdata.U_PC[ranges:ranges+idx_add,0]))
-        if self.verbose: print('Measured Ud_crowbar: ', self.TDMSdata.Ud_crow)
+        I_crowbar = signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 101)
+        I_crow = abs(np.sum(I_crowbar[ranges+idx_add:])/len(I_crowbar[ranges+idx_add:]))
+        U_remaining = abs((np.sum(self.TDMSdata.U_PC[ranges+idx_add:, 0])-self.TDMSdata.Ud_crow)/len(self.TDMSdata.U_PC[ranges+idx_add:,0]))
+        if self.verbose:
+            print('PRELIMINARY (unused): Measured Crowbar resistance: ', U_remaining/I_crow)
+            print('Measured Ud_crowbar: ', self.TDMSdata.Ud_crow)
 
-        # R_crowbar
+        # Find R_EE
+        U_EE = signal.medfilt(self.TDMSdata.U_EE, kernel_size=3)
+        self.TDMSdata.R_EE = np.max(U_EE)/self.TDMSdata.I_DCCT_MF[np.argmax(U_EE)]
+        if self.verbose:
+            print('Measured R_EE: ', self.TDMSdata.R_EE)
 
+        # R_QH & tau_QH
+        self.TDMSdata.R_QH = (np.max(self.TDMSdata.U_QH, axis=1)/np.max(self.TDMSdata.I_QH, axis=1))
+        tMax = self.TDMSdata.TimeFrame_MF[np.argmax(self.TDMSdata.I_QH, axis=1)].reshape(1, -1)
+        IQHmax = np.max(self.TDMSdata.I_QH, axis=1)
+        IQH_tau = np.exp(-1) * IQHmax
+        tTau = np.ones((len(IQH_tau),))
+        if np.any(IQHmax <1):
+            for i in range(len(IQH_tau)):
+                tTau[i] = self.TDMSdata.TimeFrame_MF[np.where(abs(self.TDMSdata.I_QH[i] - IQH_tau[i]) < 1)[0][0]]
+            tau_QH = tTau - tMax
+            self.TDMSdata.tau_QH = tau_QH[0]
+            self.TDMSdata.C_QH = self.TDMSdata.tau_QH/ self.TDMSdata.R_QH
+            if self.verbose:
+                print('Measured Total R_QH: ', self.TDMSdata.R_QH)
+                print('Measured tau_QH: ', self.TDMSdata.tau_QH)
+                print('Measured C_QH: ', self.TDMSdata.C_QH)
+                print('Measured max. U_QH: ', np.max(self.TDMSdata.U_QH,axis=1))
+        else:
+            self.TDMSdata.t_QH = 0
 
-        # R_CLIQ/R_QH
+        # R_CLIQ
+        ## TODO, see notebook
 
-    def ExtractFeatures(self):
+        return I_init
+
+    def ExtractFeatures(self, SIM= 1, TDMS=1):
         ## Extract Timing features:
         print('Timing')
         # # Extract t_PC
-        dIdt_Coil = np.zeros((self.SIMdata.I_CoilSections.shape))
-        for i in range((dIdt_Coil.shape[1])):
-            dIdt_Coil[:,i] = np.gradient(self.SIMdata.I_CoilSections[:,i], self.SIMdata.TimeFrame[:,0])
-        print('t_PC_Sim: ', self._FindFiring(dIdt_Coil, self.SIMdata.TimeFrame))
-        dIdt_Coil = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 17), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
-        print('t_PC_Meas: ', self._FindFiring(dIdt_Coil, self.TDMSdata.TimeFrame_MF, dI_tol = 10000))
+        if SIM:
+            dIdt_Coil = np.zeros((self.SIMdata.I_CoilSections.shape))
+            for i in range((dIdt_Coil.shape[1])):
+                dIdt_Coil[:,i] = np.gradient(self.SIMdata.I_CoilSections[:,i], self.SIMdata.TimeFrame[:,0])
+            t_PC_SIM = self._FindFiring(dIdt_Coil, self.SIMdata.TimeFrame)
+            print('t_PC_Sim: ', t_PC_SIM)
+        if TDMS:
+            dIdt_Coil = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 17), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
+            self.TDMSdata.t_PC = self._FindFiring(dIdt_Coil, self.TDMSdata.TimeFrame_MF, dI_tol=10000)[0]
+            print('t_PC_Meas: ', self.TDMSdata.t_PC)
         # # Extract t_QH
-        dIdt_QH = np.zeros((self.SIMdata.I_QH.shape))
-        for i in range((dIdt_QH.shape[0])):
-            dIdt_QH[i,:] = np.gradient(self.SIMdata.I_QH[i,:], self.SIMdata.TimeFrame[:,0])
-        print('t_QH_Sim: ', self._FindFiring(dIdt_QH.transpose(), self.SIMdata.TimeFrame))
-        dIdt_QH = np.zeros((self.TDMSdata.I_QH.shape))
-        for i in range((dIdt_QH.shape[0])):
-            dIdt_QH[i,:] = np.gradient(signal.medfilt(self.TDMSdata.I_QH[i,:], kernel_size = 11), self.TDMSdata.TimeFrame_MF[:,0])
-        print('t_QH_Meas: ', self._FindFiring(dIdt_QH.transpose(), self.TDMSdata.TimeFrame_MF, dI_tol=1000))
+        if SIM:
+            dIdt_QH = np.zeros((self.SIMdata.I_QH.shape))
+            for i in range((dIdt_QH.shape[0])):
+                dIdt_QH[i,:] = np.gradient(self.SIMdata.I_QH[i,:], self.SIMdata.TimeFrame[:,0])
+            t_QH_Sim = self._FindFiring(dIdt_QH.transpose(), self.SIMdata.TimeFrame)
+            print('t_QH_Sim: ', t_QH_Sim)
+        if TDMS:
+            self.TDMSdata.t_QH = self.TDMSdata.TimeFrame_MF[np.argmax(self.TDMSdata.I_QH, axis=1),0]
+            print('t_QH_Meas: ', np.transpose(self.TDMSdata.t_QH))
         # # # Extract t_CLIQ
-        dIdt_CLIQ = np.array([np.gradient(self.SIMdata.I_CLIQ[:,0], self.SIMdata.TimeFrame[:,0])]).transpose()
-        print('t_CLIQ_Sim: ', self._FindFiring(dIdt_CLIQ, self.SIMdata.TimeFrame))
-        dIdt_CLIQ = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_CLIQ[:,0], kernel_size = 11), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
-        print('t_CLIQ_Meas: ', self._FindFiring(dIdt_CLIQ, self.TDMSdata.TimeFrame_MF, dI_tol = 5000))
+        if SIM:
+            dIdt_CLIQ = np.array([np.gradient(self.SIMdata.I_CLIQ[:,0], self.SIMdata.TimeFrame[:,0])]).transpose()
+            t_CLIQ_Sim = self._FindFiring(dIdt_CLIQ, self.SIMdata.TimeFrame)
+            print('t_CLIQ_Sim: ', t_CLIQ_Sim)
+        if TDMS:
+            dIdt_CLIQ = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_CLIQ[:,0], kernel_size = 11), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
+            self.TDMSdata.t_CLIQ = self._FindFiring(dIdt_CLIQ, self.TDMSdata.TimeFrame_MF, dI_tol = 5000)
+            print('t_CLIQ_Meas: ', self.TDMSdata.t_CLIQ)
+
         # # # Extract t_EE
-        ## TODO
+        if TDMS:
+            dUdt_EE = np.array([np.gradient(signal.medfilt(self.TDMSdata.U_EE, kernel_size=11),
+                                              self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
+            self.TDMSdata.t_EE = self._FindFiring(dUdt_EE, self.TDMSdata.TimeFrame_MF, dI_tol=5000)
+            print('t_EE: ', self.TDMSdata.t_EE)
 
         ## Extract t_Quench [if applicable]
         # self.FindQuenchTDMS(0.1, 0.007)
 
         ## Extract electrical parameters
-        self.__ExtractElectricalParameters()
+        I_init = self.__ExtractElectricalParameters()
 
         ## Extract further properties
             #To be seen if in this function
-        return 0
+
+        return I_init
 
     #############################################################################################################
     ## Domain: General Functions, meant to be used from outside
@@ -995,3 +1143,23 @@ class QuenchPlanAnalysis():
                                  Voltages_Tdms[k, i2], '']
                 writer.writerow(row)
         return
+
+    def SaveSignalToCSV(self, Signal, OutputDirectory = ''):
+        if Signal in self.TDMSdata.__annotations__:
+            data = getattr(self.TDMSdata, Signal)
+            fileName = os.path.join(OutputDirectory,'Signal_'+Signal+'.csv')
+            with open(fileName, mode='w', newline='') as file:
+                writer = csv.writer(file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
+                writer.writerow(['time_vector',Signal])
+                if data.shape[0]>data.shape[1]:
+                    data = data.transpose()
+                for i in range(data.shape[1]):
+                    wr = [self.TDMSdata.TimeFrame_MF[i,0]]
+                    for j in range(data.shape[0]):
+                        wr.append(data[j,i])
+                    writer.writerow(wr)
+            print('Signal ',Signal,' saved to csv.')
+        else:
+            print('Signal could not be found in TDMS data.')
+            return
+
