@@ -25,21 +25,23 @@ testfile11 = "C:\cernbox\TEMP2\MCBY_1AP_CopperWedges_ThCool_29.xlsx"
 testfile12 = "C:\cernbox\LEDET\LEDET\MCBY_1AP_CopperWedges_ThCool\Input\MCBY_1AP_CopperWedges_ThCool_35.xlsx"
 testfile13 = "C:\cernbox\LEDET_MT\LEDET\MQXFS4b\Input\MQXFS4b_23.xlsx"
 testfile14 = "C:\cernbox\MasterThesis\MQY studies\MQY_2in1_Ref.xlsx"
+testfile15 = "C:\cernbox\LEDET\LEDET_MT\LEDET\MQXFS4b\Input\MQXFS4b_7.xlsx"
 
-ledetFolder = 'C:\\cernbox\\LEDET_OS\\'
-ledetExe = 'LEDET_v1_08_03.exe'
+
+ledetFolder = 'C:\\cernbox\\LEDET_OS\\LEDET\\'
+ledetExe = 'LEDET_v2_01_16.exe'
 
 def CompleteRun():
     start = time.time()
     a = ParametersLEDET()
-    a.readLEDETExcel(testfile13)
+    a.readLEDETExcel(testfile15)
     Sw = MinMaxSweep(a, 6)
     MagnetName = 'MQXFS4b'
 
     # ##MQXF, 6 base points
     Sw.addParameterToSweep('tau_increaseRis', 0.001, 0.2)
-    Sw.addParameterToSweep('Rcapa', 0.00001, 0.1)
     Sw.addParameterToSweep('tau_increaseRif', 0.001, 0.2)
+    Sw.addParameterToSweep('f_ro_eff_inGroup', 1.5, 2.25)
 
 
     ##13mag
@@ -88,16 +90,16 @@ def CompleteRun():
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP2\\", OffsetNumber= 42, ROXIE_File='C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-ledet-input\\MCBXV\\MCBXV_CopperWedges_All_WithIron_WithSelfField.map2d')
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP2\\", OffsetNumber=17, ROXIE_File='C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-ledet-input\\MCBY_1AP\\MCBY_1AP_CopperWedges_ThCool_All_WithIron_WithSelfField.map2d')
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\", OffsetNumber=50)
-    # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\")
+    Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\")
 
 
     end = time.time()
     print("Time:", end - start)
     print("Preparation done")
 
-    # SimNumbers = np.linspace(50,265, 216).astype(int)
+    SimNumbers = np.linspace(1,216, 216).astype(int)
 
-    # RunSimulations(ledetFolder, ledetExe, MagnetName, Simulations = SimNumbers, RunSimulations=False)
+    RunSimulations(ledetFolder, ledetExe, MagnetName, Simulations = SimNumbers, RunSimulations=False)
     # EvaluateSimulations("C:\\cernbox\\LEDET\\LEDET\\MCO\\Output\\Txt Files", 'MCO',
     #                     'C:\\cernbox\\Validation_MCDO\\Exp Data\\RCO.A56B1_2018-03-16_PM_I_A_AutoAlign.csv',
     #                     Sw, Mat=False, SkipAlign=True)
@@ -128,10 +130,10 @@ def CompleteRun():
     #                     'MCBXH_CopperWedges_ThCool',
     #                     'C:\\cernbox\\Validation_MCBX_HV\\Exp Data\\RCBXH1.R1_2018-03-11_PM_I_ARAW.csv',
     #                     Sw, Mat=False)
-    EvaluateSimulations("C:\\cernbox\\LEDET_OS\\LEDET\\MQXFS4b\\Output\\Txt Files",
-                        MagnetName,
-                        'C:\\cernbox\\MasterThesis\\Test23_ParSweepMeas.csv',
-                        Sw, Mat=False)
+    # EvaluateSimulations("C:\\cernbox\\LEDET_OS\\LEDET\\MQXFS4b\\Output\\Txt Files",
+    #                     MagnetName,
+    #                     'C:\\cernbox\\MasterThesis\\Test23_ParSweepMeas.csv',
+    #                     Sw, Mat=False)
 
 def AutomaticRun():
     LEDETFolder = 'C:\\cernbox\LEDET\\LEDET_SimulationFiles_Quench'
@@ -163,10 +165,10 @@ def testvQ():
     # RoxieFile = "C:\\cernbox\\LEDET_MT\\Field Maps\\MQXFBP2\\"
     a = ParametersLEDET()
     a.readLEDETExcel(nameFileLEDET)
-    [vQ, l, th_con_h, delta_t_h, th_con_w, delta_t_w] = a.adjust_vQ(RoxieFile)
+    l = a.adjust_vQ(RoxieFile, Transversaldelay = 'Long')
     return a
 
-def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim, verbose = True):
+def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim, verbose = False):
     Coil108 = {'C108_I1_I2':[],
                'C108_I2_I3':np.concatenate((np.linspace(29, 45, 17), np.linspace(229, 245, 17))),
                'C108_I3_I4':np.concatenate((np.linspace(46, 49, 4), np.linspace(246, 249, 4))),
@@ -231,7 +233,7 @@ def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim, verbose = True):
     # a = QuenchPlanAnalysis(0, FileName_Sim)
     # a.ProvideTurnsToCoilStructure(Coil = [Coil109,Coil110,Coil111,Coil108])
     # a.ProvideTurnsToCoilStructure(CoilCsv = 'C:\cernbox\MasterThesis\CoilStructure_MQXFPB2.csv')
-    a.ProvideTurnsToCoilStructure(CoilCsv='C:\cernbox\MasterThesis\CoilStructure_MQXFS4b.csv')
+    # a.ProvideTurnsToCoilStructure(CoilCsv='C:\cernbox\MasterThesis\CoilStructure_MQXFS4b.csv')
     # a.QuenchPlanAnalysis(Plot = 1)
     # a.FindQuenchTDMS(0.1, 0.007, Plot=1)
     # a.PlotVoltageTap(Coil = 110)
@@ -486,11 +488,12 @@ def testMP3():
     # MP3 = MP3_setup('RSS.A81B1', ParameterFile, Opts = Opts, enableQuench=0)
     # MP3.load_config('MarvinJ')
     # MP3.SetUpSimulation('FPA', 250)
-    ParameterFile = "IPQ_Circuit_Param_Table.csv"
-    MP3 = MP3_setup("RQ4.L2", ParameterFile)
-    MP3.load_config('MarvinJ')
-    MP3.SetUpSimulation('unbalancedFPA', [[250, 500]],
-                        ManualCircuit="C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-sing-input\\IPQ\\IPQ_MagnetEQ4.cir")
+    ParameterFile = "RQ_Circuit_Param_Table.csv"
+    MP3 = LibSim_setup("RQD.A45", ParameterFile)
+    MP3.load_config('mjanitsc_Openstack')
+    Currents = [[11850]]
+    MP3.SetUpSimulation('FPA', Currents, ManualStimuli = ['I_FPA_RQDF'],
+                        ManualCircuit = "/eos/home-m/mjanitsc/SWAN_projects/steam-notebooks/steam-sing-input/RQD_RQF/RQD_MagnetEQ_47mag.cir")
     return MP3
 
 # a = test_persCurrents2()
@@ -508,8 +511,8 @@ def testMP3():
 # testConsistencyChecks()
 
 
-CompareLEDETParameters("C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQ_1AP\MQ_1AP_0.xlsx",
-                       "S:\LHC\RQD_RQF\ElMaTh\LEDET\LEDET\MQ\Input\MQ_1.xlsx")
+# CompareLEDETParameters("C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQ_1AP\MQ_1AP_0.xlsx",
+#                        "S:\LHC\RQD_RQF\ElMaTh\LEDET\LEDET\MQ\Input\MQ_1.xlsx")
 # CompareLEDETParameters("C:\cernbox\MB_1.xlsx","C:\cernbox\MB_65.xlsx")
 # CompareLEDETParameters("C:\cernbox\MQXA_200.xlsx","C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_200.xlsx")
 
@@ -532,7 +535,6 @@ CompareLEDETParameters("C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-inp
 # a  = testvQ()
 
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFPB01\\HCLMQXFBT01-CR000001__A202007140955_a001(0).tdms"
-
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810151854_a026(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810111418_a003(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810111846_a008(0).tdms"
@@ -547,6 +549,9 @@ CompareLEDETParameters("C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-inp
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810171144_a039(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810161600_a033(0).tdms"
 
+# td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810121620_a019(0).tdms"
+td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810101443_tb010(0).tdms"
+
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFBP2\\Output\\Mat Files\\SimulationResults_LEDET_10.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_12.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_10.mat"
@@ -557,9 +562,16 @@ CompareLEDETParameters("C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-inp
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_3.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_12.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_26.mat"
+# sim_data = "C:\\cernbox\\LEDET\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_44.mat"
 
-# a = testQuenchPlanAnalysis(td_data, sim_data)
-# a.ExtractFeatures()
+# Ref_File = "C:\\cernbox\\MasterThesis\\MQXFS4b_Ref_0.xlsx"
+OutputDirectory = "C:\\cernbox\\TEMP_toDelete"
+
+a = testQuenchPlanAnalysis(td_data, 0)
+# a.SaveSignalToCSV("I_CLIQ",OutputDirectory=OutputDirectory)
+# a.PrepareSimulation_FromMeasurements(Ref_File, OutputDirectory, nQHseries = 2)
+
+a.ExtractFeatures(SIM=0)
 # a = testQuenchPlanAnalysis(0, sim_data)
 # a = testQuenchPlanAnalysis(td_data, 0)
 
