@@ -11,15 +11,16 @@ def RunSimulations(LEDETFolder, LEDETExe, MagnetName, Simulations = 'All', RunSi
     SimNumbers = []
 
     #1. Prepare everything
-    if(Simulations=='All'):
-        items = os.listdir(ExcelFolder)
-        for item in items:
-            if item.startswith(MagnetName) and item.endswith('.xlsx'):
-                if ".sys" not in item:
-                    num = item.replace('.xlsx', '')
-                    num = num.replace(MagnetName+'_', '')
-                    num = int(num)
-                    SimNumbers.append(num)
+    if len(Simulations)==3:
+        if Simulations =='All':
+            items = os.listdir(ExcelFolder)
+            for item in items:
+                if item.startswith(MagnetName) and item.endswith('.xlsx'):
+                    if ".sys" not in item:
+                        num = item.replace('.xlsx', '')
+                        num = num.replace(MagnetName+'_', '')
+                        num = int(num)
+                        SimNumbers.append(num)
     else:
         SimNumbers = Simulations
 
