@@ -190,7 +190,7 @@ class ParametersCOSIM:
                                t_0 = [0, 2e-5], t_end = [2e-5, 0.5], t_step_max = [[1e-5, 1e-5], [1e-5, 1e-5]],
                                relTolerance = [1e-4, None], absTolerance = [1, None], executionOrder = [1, 2],
                                executeCleanRun = [True, True], N_LEDET = 1, QuenchMagnet = 0, SimulationNumber = [0], DistinctMagnets = 1, PSPICEinitialConditions = [],
-                               convergenceElement = 'x_mag_1'):
+                               convergenceElement = 'x_mag_1', CombinedModel = 0):
 
         if ModelFolder.endswith('//'): ModelFolder.replace('//','\\')
         elif not ModelFolder.endswith('\\'): ModelFolder = ModelFolder + '\\'
@@ -201,6 +201,7 @@ class ParametersCOSIM:
         self.tEnd = t_end
         self.tStepMax = t_step_max
         # Components which are used in the Ports
+        if not convergenceElement: convergenceElement = 'x_mag_1'
         Components = ["L", "CoilSections"]  # [0] = PSpice, [1]=LEDET
         # Setting the Coilsections
         CSections = []
@@ -212,6 +213,15 @@ class ParametersCOSIM:
             CSections.append(CSectionsPerPort)
             LSections.append(Components[0] + "_" + str(i+1))
 
+        # If CoilSection-Variable is anti-symmetrical, it's assumed to represent the whole Simulation-Sections
+        split = 0
+        if len(CoilSections)>1:
+            for j in range(len(CoilSections)):
+                tempLength = len(CoilSections[j])
+                for i in range(len(CoilSections)):
+                    if len(CoilSections[i]) != tempLength:
+                        split = 1
+
         # Variables used for convergence, for now its the first magnet current and the LEDET Voltages
         if N_LEDET == 1:
             convergenceVariables = ["I(" + convergenceElement + "." + Components[0] + "_1)", "U_inductive_dynamic_" + Components[1]+"_1"]
@@ -365,6 +375,17 @@ class ParametersCOSIM:
                     json.dump(struct_total[i], f, indent=4)
                     f.write('\n')
         else:
+            SplitCount = 0
+            SectionCount = 0
+            if split:
+                lenCSections = []
+                newCSections = []
+                for i in range(len(CSections)):
+                    for j in range(len(CSections[i])):
+                        newCSections.append(CSections[i][j])
+                    lenCSections.append(len(CSections[i]))
+                CSections = newCSections
+
             for l in range(0,N_LEDET):
                 filename = os.path.join(self.nameFolderCosimModel,'LEDET_'+str(l+1),'LedetInputOutputPortDefinition.json')
                 try:
@@ -375,18 +396,33 @@ class ParametersCOSIM:
 
                 struct_total = []
                 for i in range(len(CoilSections)):  # Number of Ports
+                    if split:
+                        if i != SplitCount - SectionCount: continue
+                        if i>lenCSections[l]-1: continue
+                        SplitCount = SplitCount + 1
+                        if SplitCount == len(CSections):
+                            i = SplitCount-1
                     struct = {}
-                    struct["name"] = "Port_" + str(i+l*len(CoilSections)+1) + "_" + str(0)
+                    if SplitCount == len(CSections):
+                        struct["name"] = "Port_" + str(i + 1) + "_" + str(0)
+                    else:
+                        struct["name"] = "Port_" + str(i+l*len(CoilSections)+1) + "_" + str(0)
                     struct["components"] = []
-                    for n in range(len(CSections[i])):  # All Coilsections
-                        struct["components"].append(CSections[i][n])
+                    if split:
+                        struct["components"].append(CSections[i])
+                    else:
+                        for n in range(len(CSections[i])):  # All Coilsections
+                            struct["components"].append(CSections[i][n])
                     struct["inputs"] = []
                     for j in range(len(couplingParameterLedet[0])):  # Number of In's per respective Port
                         substruct_in = {}
                         substruct_in["couplingParameter"] = couplingParameterLedet[0][j][0]
                         substruct_in["labels"] = []
-                        for n in range(len(CSections[i])):  # Append all Coilsections as Labels to Input
-                            substruct_in["labels"].append(couplingParameterLedet[0][j][0] + "_" + CSections[i][n])
+                        if split:
+                            substruct_in["labels"].append(couplingParameterLedet[0][j][0] + "_" + CSections[i])
+                        else:
+                            for n in range(len(CSections[i])):  # Append all Coilsections as Labels to Input
+                                substruct_in["labels"].append(couplingParameterLedet[0][j][0] + "_" + CSections[i][n])
                         substruct_in["types"] = couplingParameterLedet[0][j][1]
                         struct["inputs"].append(substruct_in)
 
@@ -395,15 +431,23 @@ class ParametersCOSIM:
                         substruct_out = {}
                         substruct_out["couplingParameter"] = couplingParameterLedet[1][k][0]
                         substruct_out["labels"] = []
-                        for n in range(len(CSections[i])):  # Append all Coilsections as Labels to Output
+                        if split:
                             if (couplingParameterLedet[1][k][0] == "U"):  # If Output is U
-                                substruct_out["labels"].append("U_inductive_dynamic" + "_" + CSections[i][n])
+                                substruct_out["labels"].append("U_inductive_dynamic" + "_" + CSections[i])
                             else:  # If output is I or R
-                                substruct_out["labels"].append(couplingParameterLedet[1][k][0] + "_" + CSections[i][n])
+                                substruct_out["labels"].append(couplingParameterLedet[1][k][0] + "_" + CSections[i])
+                        else:
+                            for n in range(len(CSections[i])):  # Append all Coilsections as Labels to Output
+                                if (couplingParameterLedet[1][k][0] == "U"):  # If Output is U
+                                    substruct_out["labels"].append("U_inductive_dynamic" + "_" + CSections[i][n])
+                                else:  # If output is I or R
+                                    substruct_out["labels"].append(couplingParameterLedet[1][k][0] + "_" + CSections[i][n])
                         substruct_out["types"] = couplingParameterLedet[1][k][1]
                         struct["outputs"].append(substruct_out)
 
                     struct_total.append(struct)
+                if split:
+                    SectionCount = SectionCount + lenCSections[l]
 
                 with open(filename, 'w') as f:
                     for i in range(len(struct_total)):
@@ -433,11 +477,21 @@ class ParametersCOSIM:
             else:
                 DM = 1
                 M_add = ''
+
+            if not 'x_mag_' in convergenceElement:
+                convEl = convergenceElement
+                DM = ''
+            else: convEl = convergenceElement[:-1]
+
             for i in range(len(CoilSections)):  # Number of Ports
+                if split and len(CoilSections[n]) != len(CoilSections):
+                    if i+1 != n: continue
                 for j in range(2):  # 2Ports in PSpice
                     struct = {}
                     struct["name"] = "Port_" + str(i+n*len(CoilSections)+1) + "_" + str(j)
+                    if CombinedModel: LSections[i] = Components[0] + "_" + str(i+n*len(CoilSections)+1)
                     struct["components"] = [LSections[i]]
+                    Number = LSections[i][-1]
                     struct["inputs"] = []
                     for l in range(len(couplingParameterPSpice[j][0])):  # Number of In's
                         substruct_in = {}
@@ -447,9 +501,9 @@ class ParametersCOSIM:
                                 cPPSpice = "V"
                             else:
                                 cPPSpice = couplingParameterPSpice[j][0][l][0]
-                            substruct_in["labels"] = [cPPSpice + "_field_"+M_add + str(i+1) + "_stim"+quench]  # First Port for Field
+                            substruct_in["labels"] = [cPPSpice + "_field_"+M_add + str(Number) + "_stim"+quench]  # First Port for Field
                         else:  # Second Port with circuit
-                            substruct_in["labels"] = ["V_circuit_" +M_add+ str(i+1) + "_stim"+quench]
+                            substruct_in["labels"] = ["V_circuit_" +M_add+ str(Number) + "_stim"+quench]
                         substruct_in["types"] = couplingParameterPSpice[j][0][l][1]
                         struct["inputs"].append(substruct_in)
 
@@ -463,9 +517,9 @@ class ParametersCOSIM:
                             else:
                                 cPPSpice = couplingParameterPSpice[j][1][k][0]
                             substruct_out["labels"] = [
-                                cPPSpice + "(x_mag_" + str(DM)+ "." + Components[0] + "_" + str(i+1) + ")"]
+                                cPPSpice + "("+ convEl + str(DM)+ "." + Components[0] + "_" + str(Number) + ")"]
                         else:
-                            substruct_out["labels"] = ["V(x_mag_" + str(DM)+ "." + str(i+1) + "_v_l_diff)"]
+                            substruct_out["labels"] = ["V("+convEl + str(DM)+ "." + str(Number) + "_v_l_diff)"]
                         substruct_out["types"] = couplingParameterPSpice[j][1][k][1]
                         struct["outputs"].append(substruct_out)
 
@@ -621,6 +675,11 @@ class ParametersCOSIM:
         a.Options.flag_generateReport = 0
         a.Options.flag_saveMatFile = 0
         a.Options.flag_automaticRefinedTimeStepping = 0
+
+        if 'R_el_ht' not in a.Variables.variableToSaveTxt:
+            a.Variables.variableToSaveTxt = np.append(a.Variables.variableToSaveTxt, 'R_el_ht')
+            a.Variables.typeVariableToSaveTxt = np.append(a.Variables.typeVariableToSaveTxt, 1)
+
         a.writeFileLEDET(file, verbose = False)
 
         TurnNumber = max(a.Inputs.HalfTurnToInductanceBlock)
@@ -642,8 +701,9 @@ class ParametersCOSIM:
             LEDET_exe = LEDET_exe.replace('\ ', '\\')
             idx = self.__findOccurrences(LEDET_exe, '\\')[-1]
             stub = LEDET_exe[:idx]+'\\'
-        if Destination: fileName = os.path.join(Destination,'COSIM_nb_Batch.bat')
-        else: fileName = 'COSIM_nb_Batch.bat'
+        batch_name = f'COSIM_nb_Batch.bat'
+        if Destination: fileName = os.path.join(Destination,batch_name)
+        else: fileName = batch_name
         if Append: fN = open(fileName, 'a+')
         else: fN = open(fileName, 'w+')
         txt_stub = "java -jar " + COSIMexe
@@ -673,3 +733,17 @@ class ParametersCOSIM:
             print(len(COSIMfolder), ' simulations to batch-file appended.')
         print('Time stamp: ' + str(currentDT))
 
+    def StampBatch(self, Stamp, Folder, Destination):
+        bat_core = 'COSIM_nb_Batch'
+        bat_ext = ".bat"
+        new_bat_name = f"{bat_core}_{Stamp}{bat_ext}"
+        new_bat_path = os.path.join(Destination, new_bat_name)
+        if os.path.exists(new_bat_path):
+            os.remove(new_bat_path)
+        os.rename(os.path.join(Destination, f"{bat_core}{bat_ext}"), new_bat_path)
+        if Folder != '':
+            model_folder = os.path.join(Destination, Folder)
+            new_bat_loc = os.path.join(model_folder, new_bat_name)
+            if os.path.exists(new_bat_loc):
+                os.remove(new_bat_loc)
+            shutil.move(new_bat_path, model_folder)
\ No newline at end of file
